import logging
import sys
import unittest

from nose2 import events, util

log = logging.getLogger(__name__)

__unittest = True

#
# Layer suite class
#


class LayerSuite(unittest.BaseTestSuite):
    def __init__(self, session, tests=(), layer=None):
        super().__init__(tests)
        self.layer = layer
        self.wasSetup = False
        self.session = session

    def run(self, result):
        self.handle_previous_test_teardown(result)
        if not self._safeMethodCall(self.setUp, result):
            return
        try:
            for test in self:
                if result.shouldStop:
                    break
                self._safeMethodCall(self.setUpTest, result, test)
                try:
                    test(result)
                finally:
                    self._safeMethodCall(self.tearDownTest, result, test)
        finally:
            if self.wasSetup:
                self._safeMethodCall(self.tearDown, result)

    def handle_previous_test_teardown(self, result):
        prev = getattr(result, "_previousTestClass", None)
        if prev is None:
            return
        layer_attr = getattr(prev, "layer", None)
        if isinstance(layer_attr, LayerSuite):
            return
        try:
            suite_obj = unittest.suite.TestSuite()
            suite_obj._tearDownPreviousClass(None, result)
            suite_obj._handleModuleTearDown(result)
        finally:
            result._previousTestClass = None

    def setUp(self):
        if self.layer is None:
            return

        event = events.StartLayerSetupEvent(self.layer)
        self.session.hooks.startLayerSetup(event)

        setup = self._getBoundClassmethod(self.layer, "setUp")

        if setup:
            setup()
        self.wasSetup = True

        event = events.StopLayerSetupEvent(self.layer)
        self.session.hooks.stopLayerSetup(event)

    def setUpTest(self, test):
        if self.layer is None:
            return
        # skip suites, to ensure test setup only runs once around each test
        # even for sub-layer suites inside this suite.
        try:
            iter(test)
        except TypeError:
            # ok, not a suite
            pass
        else:
            # suite-like enough for skipping
            return
        if getattr(test, "_layer_wasSetUp", False):
            return

        event = events.StartLayerSetupTestEvent(self.layer, test)
        self.session.hooks.startLayerSetupTest(event)

        self._allLayers(test, "testSetUp")
        test._layer_wasSetUp = True

        event = events.StopLayerSetupTestEvent(self.layer, test)
        self.session.hooks.stopLayerSetupTest(event)

    def tearDownTest(self, test):
        if self.layer is None:
            return
        if not getattr(test, "_layer_wasSetUp", None):
            return

        event = events.StartLayerTeardownTestEvent(self.layer, test)
        self.session.hooks.startLayerTeardownTest(event)

        self._allLayers(test, "testTearDown", reverse=True)

        event = events.StopLayerTeardownTestEvent(self.layer, test)
        self.session.hooks.stopLayerTeardownTest(event)
        delattr(test, "_layer_wasSetUp")

    def tearDown(self):
        if self.layer is None:
            return

        event = events.StartLayerTeardownEvent(self.layer)
        self.session.hooks.startLayerTeardown(event)
        teardown = self._getBoundClassmethod(self.layer, "tearDown")
        if teardown:
            teardown()
        event = events.StopLayerTeardownEvent(self.layer)
        self.session.hooks.stopLayerTeardown(event)

    def _safeMethodCall(self, method, result, *args):
        try:
            method(*args)
            return True
        except KeyboardInterrupt:
            raise
        except BaseException:
            result.addError(self, sys.exc_info())
            return False

    def _allLayers(self, test, method, reverse=False):
        done = set()
        all_lys = util.ancestry(self.layer)
        if reverse:
            all_lys = [reversed(lys) for lys in reversed(all_lys)]
        for lys in all_lys:
            for layer in lys:
                if layer in done:
                    continue
                self._inLayer(layer, test, method)
                done.add(layer)

    def _inLayer(self, layer, test, method):
        meth = self._getBoundClassmethod(layer, method)
        if meth:
            if util.num_expected_args(meth) > 1:
                meth(test)
            else:
                meth()

    def _getBoundClassmethod(self, cls, method):
        """
        Use instead of :func:`getattr` to get only classmethods explicitly
        defined on ``cls`` (not methods inherited from ancestors)
        """
        descriptor = cls.__dict__.get(method, None)
        if descriptor:
            if not isinstance(descriptor, classmethod):
                raise TypeError(
                    "The %s method on a layer must be a classmethod." % method
                )
            bound_method = descriptor.__get__(None, cls)
            return bound_method
        else:
            return None



import time

# statement run at import time should be covered
foo = 1

with open("/tmp/showme", "a") as f:
    f.write("\n\nbeta" + str(time.time()))


raise SystemExit


# so should an ordinary function body
def func():
    return 2



from nose2 import events, loader, session
from nose2.tests._common import TestCase


class TestPluggableTestLoader(TestCase):
    def setUp(self):
        self.session = session.Session()
        self.loader = loader.PluggableTestLoader(self.session)

    def test_failed_load_tests_exception(self):
        suite = self.loader.failedLoadTests("test", RuntimeError("err"))
        tc = suite._tests[0]
        with self.assertRaises(RuntimeError) as cm:
            tc.test()
        self.assertEqual(cm.exception.args, ("err",))

    def test_failed_load_tests_exc_info(self):
        suite = self.loader.failedLoadTests(
            "test", (RuntimeError, RuntimeError("err"), None)
        )
        tc = suite._tests[0]
        with self.assertRaises(RuntimeError) as cm:
            tc.test()
        self.assertEqual(cm.exception.args, ("err",))

    def test_load_from_module_calls_hook(self):
        self.session.hooks.register("loadTestsFromModule", FakePlugin())
        evt = events.LoadFromModuleEvent(self.loader, "some_module")
        self.session.hooks.loadTestsFromModule(evt)
        self.assertTrue(
            evt.fakeLoadFromModule, "FakePlugin.loadTestsFromModule() was not called"
        )

    def test_load_from_name_calls_hook(self):
        self.session.hooks.register("loadTestsFromName", FakePlugin())
        evt = events.LoadFromNameEvent(self.loader, "some_name", "some_module")
        self.session.hooks.loadTestsFromName(evt)
        self.assertTrue(
            evt.fakeLoadFromName, "FakePlugin.fakeLoadFromName() was not called"
        )

    def test_load_from_names_calls_hook(self):
        self.session.hooks.register("loadTestsFromNames", FakePlugin())
        evt = events.LoadFromNamesEvent(self.loader, ["some_name"], "some_module")
        self.session.hooks.loadTestsFromNames(evt)
        self.assertTrue(
            evt.fakeLoadFromNames, "FakePlugin.fakeLoadFromNames() was not called"
        )

    def test_loader_from_names_calls_module_hook(self):
        fake_plugin = FakePlugin()
        self.session.hooks.register("loadTestsFromModule", fake_plugin)
        self.loader.loadTestsFromNames([], "some_module")
        self.assertTrue(
            fake_plugin.fakeLoadFromModule,
            "FakePlugin.loadTestsFromModule() was not called",
        )

    def test_loader_from_names_calls_name_hook(self):
        fake_plugin = FakePlugin()
        self.session.hooks.register("loadTestsFromName", fake_plugin)
        self.loader.loadTestsFromNames(["some_name"])
        self.assertTrue(
            fake_plugin.fakeLoadFromName,
            "FakePlugin.loadTestsFromName() was not called",
        )

    def test_loader_from_names_calls_names_hook(self):
        fake_plugin = FakePlugin()
        self.session.hooks.register("loadTestsFromNames", fake_plugin)
        self.loader.loadTestsFromNames(["some_name"])
        self.assertTrue(
            fake_plugin.fakeLoadFromNames,
            "FakePlugin.loadTestsFromNames() was not called",
        )


class FakePlugin:
    def __init__(self):
        self.fakeLoadFromModule = False
        self.fakeLoadFromName = False
        self.fakeLoadFromNames = False

    def loadTestsFromModule(self, event):
        event.fakeLoadFromModule = True
        self.fakeLoadFromModule = True

    def loadTestsFromName(self, event):
        event.fakeLoadFromName = True
        self.fakeLoadFromName = True

    def loadTestsFromNames(self, event):
        event.fakeLoadFromNames = True
        self.fakeLoadFromNames = True



import unittest

STATE = {}


class L1:
    @classmethod
    def setUp(cls):
        STATE["L1"] = "setup"

    @classmethod
    def tearDown(cls):
        del STATE["L1"]


class L2:
    @classmethod
    def setUp(cls):
        STATE["L2"] = "setup"

    @classmethod
    def tearDown(cls):
        del STATE["L2"]


class LayerAndAttributesA(unittest.TestCase):
    layer = L1
    a = 1

    def test(self):
        self.assertEqual(STATE.get("L1"), "setup")


class LayerAndAttributesB(unittest.TestCase):
    layer = L2
    b = 1

    def test(self):
        self.assertEqual(STATE.get("L2"), "setup")



import logging

from nose2.tools import such

from .common import NormalTest, NormalTestTwo, UniqueResource  # noqa: F401

log = logging.getLogger(__name__)


with such.A("system with setup") as it:

    @it.has_setup
    def setup():
        log.info("Called setup in such test")
        it.unique_resource = UniqueResource()
        it.unique_resource.lock()

    @it.has_teardown
    def teardown():
        log.info("Called teardown in such test")
        it.unique_resource.unlock()

    @it.should("do something")
    def test(case):
        it.assertTrue(it.unique_resource.used)


it.createTests(globals())



"""
Load tests from parameterized functions and methods.

This plugin implements :func:`getTestCaseNames`,
:func:`loadTestsFromModule`, and :func:`loadTestsFromName` to support
loading tests from parameterized test functions and methods.

To parameterize a function or test case method, use :func:`nose2.tools.params`.

To address a particular parameterized test via a command-line test name,
append a colon (':') followed by the index (*starting from 1*) of the
case you want to execute.

Such And The Parameters Plugin
------------------------------

The parameters plugin can work with the Such DSL, as long as the first argument
of the test function is the "case" argument, followed by the other parameters::

    from nose2.tools import such
    from nose2.tools.params import params

    with such.A('foo') as it:
        @it.should('do bar')
        @params(1,2,3)
        def test(case, bar):
            case.assertTrue(isinstance(bar, int))

        @it.should('do bar and extra')
        @params((1, 2), (3, 4) ,(5, 6))
        def testExtraArg(case, bar, foo):
            case.assertTrue(isinstance(bar, int))
            case.assertTrue(isinstance(foo, int))

    it.createTests(globals())

"""
# This module contains some code copied from unittest2 and other code
# developed in reference to unittest2.
# unittest2 is Copyright (c) 2001-2010 Python Software Foundation; All
# Rights Reserved. See: http://docs.python.org/license.html

import functools
import logging
import sys
import types
import unittest

from nose2 import exceptions, util
from nose2.events import Plugin
from nose2.plugins.loader.testclasses import MethodTestCase

log = logging.getLogger(__name__)
__unittest = True


class ParamsFunctionCase(unittest.FunctionTestCase):
    def __init__(self, name, func, **args):
        self._funcName = name
        unittest.FunctionTestCase.__init__(self, func, **args)

    def __repr__(self):
        return self._funcName

    id = __str__ = __repr__


class Parameters(Plugin):

    """Loader plugin that loads parameterized tests"""

    alwaysOn = True
    configSection = "parameters"

    def registerInSubprocess(self, event):
        event.pluginClasses.append(self.__class__)

    def getTestCaseNames(self, event):
        """Generate test case names for all parameterized methods"""
        log.debug("getTestCaseNames %s", event)
        for name, method in util.iter_attrs(event.testCase):
            if not event.isTestMethod(name):
                continue
            paramList = getattr(method, "paramList", None)
            if paramList is None:
                continue
            # exclude this method from normal collection
            event.excludedNames.append(name)
            # generate the methods to be loaded by the testcase loader
            self._generate(event, name, method, event.testCase)

    def getTestMethodNames(self, event):
        return self.getTestCaseNames(event)

    def loadTestsFromModule(self, event):
        """Load tests from parameterized test functions in the module"""
        module = event.module

        def is_test(obj):
            return obj.__name__.startswith(self.session.testMethodPrefix) and hasattr(
                obj, "paramList"
            )

        tests = []
        for _, obj in util.iter_attrs(module):
            if isinstance(obj, types.FunctionType) and is_test(obj):
                tests.extend(self._generateFuncTests(obj))
        event.extraTests.extend(tests)

    def loadTestsFromName(self, event):
        """Load parameterized test named on command line"""
        original_name = name = event.name
        module = event.module
        try:
            result = util.test_from_name(name, module)
        except (AttributeError, ImportError):
            event.handled = True
            return event.loader.failedLoadTests(name, sys.exc_info())
        if result is None:
            # we can't find it - let the default case handle it
            return

        parent, obj, fqname, index = result
        if not hasattr(obj, "paramList"):
            return

        if (
            index is None
            and not isinstance(parent, type)
            and not isinstance(obj, types.FunctionType)
        ):
            log.debug("Don't know how to load parameterized tests from %s", obj)
            return

        if (
            parent
            and isinstance(parent, type)
            and issubclass(parent, unittest.TestCase)
        ):
            # parameterized method in test case
            names = self._generate(event, obj.__name__, obj, parent)
            tests = [parent(n) for n in names]
        elif parent and isinstance(parent, type):
            # parameterized method in test class
            names = self._generate(event, obj.__name__, obj, parent)
            tests = [MethodTestCase(parent)(name) for name in names]
        else:
            # parameterized func
            tests = list(self._generateFuncTests(obj))

        if index is not None:
            try:
                tests = [tests[index - 1]]
            except IndexError:
                raise exceptions.TestNotFoundError(original_name)

        suite = event.loader.suiteClass()
        suite.addTests(tests)
        event.handled = True
        return suite

    def _generate(self, event, name, method, testCaseClass):
        names = []
        for index, argSet in enumerate_params(method.paramList):
            method_name = util.name_from_args(name, index, argSet)
            if not hasattr(testCaseClass, method_name):
                # not already generated
                def _method(self, method=method, argSet=argSet):
                    return method(self, *argSet)

                _method = functools.update_wrapper(_method, method)
                delattr(_method, "paramList")
                setattr(testCaseClass, method_name, _method)
            names.append(method_name)
        return names

    def _generateFuncTests(self, obj):
        args = {}
        setUp = getattr(obj, "setUp", None)
        tearDown = getattr(obj, "tearDown", None)
        if setUp is not None:
            args["setUp"] = setUp
        if tearDown is not None:
            args["tearDown"] = tearDown
        for index, argSet in enumerate_params(obj.paramList):

            def func(argSet=argSet, obj=obj):
                return obj(*argSet)

            func = functools.update_wrapper(func, obj)
            delattr(func, "paramList")
            name = f"{obj.__module__}.{obj.__name__}"
            func_name = util.name_from_args(name, index, argSet)
            yield util.transplant_class(ParamsFunctionCase, obj.__module__)(
                func_name, func, **args
            )


def enumerate_params(paramList):
    for index, argSet in enumerate(paramList):
        if not isinstance(argSet, tuple):
            argSet = (argSet,)
        yield index, argSet



from nose2 import result, session
from nose2.tests._common import TestCase


class TestPluggableTestResult(TestCase):
    def setUp(self):
        self.session = session.Session()
        self.result = result.PluggableTestResult(self.session)

    def test_skip_reason_not_discarded(self):
        class Test(TestCase):
            def test(self):
                pass

        plugin = FakePlugin()
        self.session.hooks.register("testOutcome", plugin)
        self.result.addSkip(Test("test"), "because")
        self.assertEqual(plugin.reason, "because")


class FakePlugin:
    def testOutcome(self, event):
        self.reason = event.reason



"""
Load tests from generators.

This plugin implements :func:`loadTestFromTestCase`,
:func:`loadTestsFromName` and :func:`loadTestFromModule` to enable
loading tests from generators.

Generators may be functions or methods in test cases. In either case,
they must yield a callable and arguments for that callable once for
each test they generate. The callable and arguments may all be in one
tuple, or the arguments may be grouped into a separate tuple::

  def test_gen():
      yield check, 1, 2
      yield check, (1, 2)

To address a particular generated test via a command-line test name,
append a colon (':') followed by the index (*starting from 1*) of the
generated case you want to execute.

"""
# This module contains some code copied from unittest2 and other code
# developed in reference to unittest2.
# unittest2 is Copyright (c) 2001-2010 Python Software Foundation; All
# Rights Reserved. See: http://docs.python.org/license.html

import functools
import inspect
import logging
import sys
import types
import unittest

from nose2 import exceptions, util
from nose2.events import Plugin

log = logging.getLogger(__name__)
__unittest = True


class Generators(Plugin):

    """Loader plugin that loads generator tests"""

    alwaysOn = True
    configSection = "generators"

    def registerInSubprocess(self, event):
        event.pluginClasses.append(self.__class__)

    def unpack(self, generator):
        for index, func_args in enumerate(generator):
            try:
                func, args = func_args
                if not isinstance(args, tuple):
                    args = (args,)
                yield index, (func, args)
            except ValueError:
                func, args = func_args[0], func_args[1:]
                yield index, (func, args)

    def loadTestsFromTestCase(self, event):
        """Load generator tests from test case"""
        log.debug("loadTestsFromTestCase %s", event.testCase)
        testCaseClass = event.testCase
        for name, method in util.iter_attrs(testCaseClass):
            if not inspect.isgeneratorfunction(method):
                continue
            if not name.startswith(self.session.testMethodPrefix):
                continue
            instance = testCaseClass(name)
            event.extraTests.extend(
                self._testsFromGenerator(event, name, method(instance), testCaseClass)
            )

    def loadTestsFromTestClass(self, event):
        testCaseClass = event.testCase
        for name, method in util.iter_attrs(testCaseClass):
            if not inspect.isgeneratorfunction(method):
                continue
            if not name.startswith(self.session.testMethodPrefix):
                continue
            instance = testCaseClass()
            event.extraTests.extend(
                self._testsFromGeneratorMethod(event, name, method, instance)
            )

    def getTestCaseNames(self, event):
        """Get generator test case names from test case class"""
        log.debug("getTestCaseNames %s", event.testCase)
        for name, method in util.iter_attrs(event.testCase):
            if not inspect.isgeneratorfunction(method):
                continue
            if not event.isTestMethod(name):
                continue
            event.excludedNames.append(name)

    def getTestMethodNames(self, event):
        return self.getTestCaseNames(event)

    def loadTestsFromName(self, event):
        """Load tests from generator named on command line"""
        original_name = name = event.name
        module = event.module
        try:
            result = util.test_from_name(name, module)
        except (AttributeError, ImportError):
            event.handled = True
            return event.loader.failedLoadTests(name, sys.exc_info())
        if result is None:
            # we can't find it - let the default case handle it
            return

        parent, obj, name, index = result
        if not inspect.isgeneratorfunction(obj):
            return

        if (
            index is None
            and not isinstance(parent, type)
            and not isinstance(obj, types.FunctionType)
        ):
            log.debug("Don't know how to load generator tests from %s", obj)
            return

        if (
            parent
            and isinstance(parent, type)
            and issubclass(parent, unittest.TestCase)
        ):
            # generator method in test case
            instance = parent(obj.__name__)
            tests = list(
                self._testsFromGenerator(event, obj.__name__, obj(instance), parent)
            )
        elif parent and isinstance(parent, type):
            # generator method in test class
            method = obj
            instance = parent()
            tests = list(self._testsFromGeneratorMethod(event, name, method, instance))
        else:
            # generator func
            tests = list(self._testsFromGeneratorFunc(event, obj))

        if index is not None:
            try:
                tests = [tests[index - 1]]
            except IndexError:
                raise exceptions.TestNotFoundError(original_name)

        suite = event.loader.suiteClass()
        suite.addTests(tests)
        event.handled = True
        return suite

    def loadTestsFromModule(self, event):
        """Load tests from generator functions in a module"""
        module = event.module

        def is_test(obj):
            return obj.__name__.startswith(
                self.session.testMethodPrefix
            ) and inspect.isgeneratorfunction(obj)

        tests = []
        for _, obj in util.iter_attrs(module):
            if isinstance(obj, types.FunctionType) and is_test(obj):
                tests.extend(self._testsFromGeneratorFunc(event, obj))
        event.extraTests.extend(tests)

    def _testsFromGenerator(self, event, name, generator, testCaseClass):
        try:
            for index, (func, args) in self.unpack(generator):
                method_name = util.name_from_args(name, index, args)
                setattr(testCaseClass, method_name, None)
                instance = testCaseClass(method_name)
                delattr(testCaseClass, method_name)

                def method(func=func, args=args):
                    return func(*args)

                method = functools.update_wrapper(method, func)
                setattr(instance, method_name, method)
                yield instance
        except Exception as e:
            test_name = "{}.{}.{}".format(
                testCaseClass.__module__,
                testCaseClass.__name__,
                name,
            )
            yield event.loader.failedLoadTests(test_name, e)

    def _testsFromGeneratorFunc(self, event, obj):
        extras = list(obj())
        name = f"{obj.__module__}.{obj.__name__}"
        args = {}
        setUp = getattr(obj, "setUp", None)
        tearDown = getattr(obj, "tearDown", None)
        if setUp is not None:
            args["setUp"] = setUp
        if tearDown is not None:
            args["tearDown"] = tearDown

        def createTest(name):
            return util.transplant_class(GeneratorFunctionCase, obj.__module__)(
                name, **args
            )

        yield from self._testsFromGenerator(event, name, extras, createTest)

    def _testsFromGeneratorMethod(self, event, name, method, instance):
        extras = list(method(instance))
        name = "{}.{}.{}".format(
            instance.__class__.__module__,
            instance.__class__.__name__,
            method.__name__,
        )
        args = {}
        setUp = getattr(instance, "setUp", None)
        tearDown = getattr(instance, "tearDown", None)
        if setUp is not None:
            args["setUp"] = setUp
        if tearDown is not None:
            args["tearDown"] = tearDown

        def createTest(name):
            return util.transplant_class(
                GeneratorMethodCase(instance.__class__), instance.__class__.__module__
            )(name, **args)

        yield from self._testsFromGenerator(event, name, extras, createTest)


class GeneratorFunctionCase(unittest.FunctionTestCase):
    def __init__(self, name, **args):
        self._funcName = name
        unittest.FunctionTestCase.__init__(self, None, **args)

    _testFunc = property(
        lambda self: getattr(self, self._funcName), lambda self, func: None
    )

    def __repr__(self):
        return self._funcName

    id = __str__ = __repr__


def GeneratorMethodCase(cls):
    class _GeneratorMethodCase(GeneratorFunctionCase):
        if util.has_class_fixtures(cls):

            @classmethod
            def setUpClass(klass):
                if hasattr(cls, "setUpClass"):
                    cls.setUpClass()

            @classmethod
            def tearDownClass(klass):
                if hasattr(cls, "tearDownClass"):
                    cls.tearDownClass()

    # XXX retain original class name
    _GeneratorMethodCase.__name__ = cls.__name__
    return _GeneratorMethodCase



import os
import sys
import unittest

from nose2 import util
from nose2.tests._common import TestCase


class UtilTests(TestCase):
    _RUN_IN_TEMP = True

    def test_ensure_importable(self):
        test_dir = os.path.join(self._work_dir, "test_dir")
        # Make sure test data is suitable for the test
        self.assertNotIn(test_dir, sys.path)
        util.ensure_importable(test_dir)
        self.assertEqual(test_dir, sys.path[0])

    def test_testcase_test_name(self):
        test = UtilTests("test_ensure_importable")
        self.assertEqual(
            test.id(), "nose2.tests.unit.test_util.UtilTests.test_ensure_importable"
        )
        self.assertEqual(
            util.test_name(test),
            "nose2.tests.unit.test_util.UtilTests.test_ensure_importable",
        )

    def test_subtest_test_name(self):
        from unittest.case import _SubTest

        sentinel = getattr(unittest.case, "_subtest_msg_sentinel", None)
        test = UtilTests("test_ensure_importable")
        test = _SubTest(test, sentinel, {"i": 1, "j": 2})
        self.assertEqual(
            test.id(),
            "nose2.tests.unit.test_util.UtilTests.test_ensure_importable (i=1, j=2)",
        )
        self.assertEqual(
            util.test_name(test),
            "nose2.tests.unit.test_util.UtilTests.test_ensure_importable",
        )


class HasClassFixturesTests(TestCase):
    def test_unittest_testcase(self):
        C = unittest.TestCase
        self.assertFalse(util.has_class_fixtures(C))
        self.assertFalse(util.has_class_fixtures(C()))

    def test_derived_testcase(self):
        class C(unittest.TestCase):
            def runTest(self):
                pass

        self.assertFalse(util.has_class_fixtures(C))
        self.assertFalse(util.has_class_fixtures(C()))

    def test_testcase_with_setup(self):
        class C(unittest.TestCase):
            @classmethod
            def setUpClass(cls):
                pass

            def runTest(self):
                pass

        self.assertTrue(util.has_class_fixtures(C))
        self.assertTrue(util.has_class_fixtures(C()))

    def test_testcase_with_teardown(self):
        class C(unittest.TestCase):
            @classmethod
            def tearDownClass(cls):
                pass

            def runTest(self):
                pass

        self.assertTrue(util.has_class_fixtures(C))
        self.assertTrue(util.has_class_fixtures(C()))

    def test_derived_derived_testcase(self):
        class C(unittest.TestCase):
            @classmethod
            def setUpClass(cls):
                pass

            @classmethod
            def tearDownClass(cls):
                pass

        class D(C):
            def runTest(self):
                pass

        self.assertTrue(util.has_class_fixtures(D))
        self.assertTrue(util.has_class_fixtures(D()))


class HasModuleFixturesTests(TestCase):
    def test_module_without_fixtures(self):
        class M:
            pass

        M.__name__ = "nose2.foo.bar"

        class C(unittest.TestCase):
            def runTest(self):
                pass

        C.__module__ = M.__name__
        m = M()
        m.C = C
        sys.modules[M.__name__] = m
        try:
            self.assertFalse(util.has_module_fixtures(C))
            self.assertFalse(util.has_module_fixtures(C()))
        finally:
            del sys.modules[M.__name__]

    def test_module_with_setup(self):
        class M:
            pass

        M.__name__ = "nose2.foo.bar"

        class C(unittest.TestCase):
            def runTest(self):
                pass

        C.__module__ = M.__name__
        m = M()
        m.C = C
        m.setUpModule = lambda: None
        sys.modules[M.__name__] = m
        try:
            self.assertTrue(util.has_module_fixtures(C))
            self.assertTrue(util.has_module_fixtures(C()))
        finally:
            del sys.modules[M.__name__]

    def test_module_with_teardown(self):
        class M:
            pass

        M.__name__ = "nose2.foo.bar"

        class C(unittest.TestCase):
            def runTest(self):
                pass

        C.__module__ = M.__name__
        m = M()
        m.C = C
        m.tearDownModule = lambda: None
        sys.modules[M.__name__] = m
        try:
            self.assertTrue(util.has_module_fixtures(C))
            self.assertTrue(util.has_module_fixtures(C()))
        finally:
            del sys.modules[M.__name__]



DEFAULT_PLUGINS = (
    "nose2.plugins.loader.discovery",
    "nose2.plugins.loader.testcases",
    "nose2.plugins.loader.functions",
    "nose2.plugins.loader.testclasses",
    "nose2.plugins.loader.generators",
    "nose2.plugins.loader.parameters",
    "nose2.plugins.loader.loadtests",
    "nose2.plugins.dundertest",
    "nose2.plugins.coverage",
    "nose2.plugins.result",
    "nose2.plugins.logcapture",
    "nose2.plugins.buffer",
    "nose2.plugins.failfast",
    "nose2.plugins.debugger",
    "nose2.plugins.prettyassert",
)



import logging

from nose2 import events, result, session
from nose2.plugins import debugger
from nose2.tests._common import TestCase


class NullHandler(logging.Handler):
    def emit(self, record):
        pass


class StubPdb:
    def __init__(self):
        self.called = False
        self.tb = None

    def post_mortem(self, tb):
        self.called = True
        self.tb = tb


class NoInteraction(events.Plugin):
    def beforeInteraction(self, event):
        event.handled = True
        return False


class TestDebugger(TestCase):
    tags = ["unit"]

    def setUp(self):
        self.session = session.Session()
        self.plugin = debugger.Debugger(session=self.session)
        self.result = result.PluggableTestResult(self.session)

        class Test(TestCase):
            def test(self):
                pass

            def test_err(self):
                raise Exception("oops")

            def test_fail(self):
                assert False

        self.case = Test

        self.pdb = self.plugin.pdb
        self.plugin.pdb = StubPdb()

        self.plugin.register()

        super(TestCase, self).setUp()

    def tearDown(self):
        self.plugin.pdb = self.pdb
        super(TestCase, self).tearDown()

    def test_does_not_call_pdb_on_success(self):
        test = self.case("test")
        test(self.result)
        assert not self.plugin.pdb.called, "pdb was called on success"

    def test_does_call_pdb_on_error(self):
        test = self.case("test_err")
        test(self.result)
        assert self.plugin.pdb.called, "pdb was not called on error"

    def test_does_call_pdb_on_failure(self):
        test = self.case("test_fail")
        test(self.result)
        assert self.plugin.pdb.called, "pdb was not called on failure"

    def test_does_not_call_pdb_on_failure_if_config_set(self):
        self.plugin.errorsOnly = True
        test = self.case("test_fail")
        test(self.result)
        assert (
            not self.plugin.pdb.called
        ), "pdb was called on failure when errorsOnly set"

    def test_other_plugins_can_prevent_interaction(self):
        # prevent 'no logger for x' warnings
        debugger.log.addHandler(NullHandler())
        nono = NoInteraction(session=self.session)
        nono.register()
        test = self.case("test_err")
        test(self.result)
        assert (
            not self.plugin.pdb.called
        ), "pdb was called despite beforeInteraction returning False"



import os.path
import shutil
import tempfile
import unittest


class Test(unittest.TestCase):
    def setUp(self):
        super().setUp()

        self.temp_dir = tempfile.mkdtemp()

    def tearDown(self):
        super().tearDown()

        shutil.rmtree(self.temp_dir, ignore_errors=True)

    def test_chdir(self):
        os.chdir(self.temp_dir)



import unittest

STATE = {}


class Base:
    @classmethod
    def setUp(cls):
        STATE["base"] = "setup"

    @classmethod
    def tearDown(cls):
        del STATE["base"]


class LayerA(Base):
    position = 1

    @classmethod
    def setUp(cls):
        STATE["layerA"] = "setup"

    @classmethod
    def tearDown(cls):
        del STATE["layerA"]

    @classmethod
    def testSetUp(cls, test):
        STATE["layerA.test"] = "setup"
        # print "->", STATE, test

    @classmethod
    def testTearDown(cls, test):
        # print "<-", STATE, test
        del STATE["layerA.test"]


class LayerA_1(LayerA):
    position = 0

    @classmethod
    def setUp(cls):
        STATE["layerA_1"] = "setup"

    @classmethod
    def tearDown(cls):
        del STATE["layerA_1"]


class LayerB(LayerA):
    position = 2

    @classmethod
    def setUp(cls):
        STATE["layerB"] = "setup"

    @classmethod
    def tearDown(cls):
        del STATE["layerB"]


class LayerB_1(LayerB):
    position = 0

    @classmethod
    def setUp(cls):
        STATE["layerB_1"] = "setup"

    @classmethod
    def tearDown(cls):
        del STATE["layerB_1"]


class LayerC(LayerB, LayerA):
    position = 1

    @classmethod
    def setUp(cls):
        STATE["layerC"] = "setup"

    @classmethod
    def tearDown(cls):
        del STATE["layerC"]


class LayerD(Base):
    position = 0

    @classmethod
    def setUp(cls):
        STATE["layerD"] = "setup"

    @classmethod
    def tearDown(cls):
        del STATE["layerD"]


class Outer(unittest.TestCase):
    layer = Base

    def test(self):
        self.assertEqual(STATE.get("base"), "setup")


class InnerA(unittest.TestCase):
    layer = LayerA

    def setUp(self):
        STATE["innerA.test"] = "setup"

    def tearDown(self):
        del STATE["innerA.test"]

    def test(self):
        expect = {
            "base": "setup",
            "layerA": "setup",
            "innerA.test": "setup",
            "layerA.test": "setup",
        }
        for k, v in expect.items():
            self.assertEqual(STATE.get(k), v)


class InnerA_1(unittest.TestCase):
    layer = LayerA_1

    def test(self):
        expect = {
            "base": "setup",
            "layerA": "setup",
            "layerA_1": "setup",
            "layerA.test": "setup",
        }
        for k, v in expect.items():
            self.assertEqual(STATE.get(k), v)


class InnerB(unittest.TestCase):
    layer = LayerB

    def setUp(self):
        STATE["innerB.test"] = "setup"

    def tearDown(self):
        STATE["innerB.test"] = "tearDown"


class InnerB_1(unittest.TestCase):
    layer = LayerB_1

    def test(self):
        expect = {"base": "setup", "layerB": "setup", "layerB_1": "setup"}
        for k, v in expect.items():
            self.assertEqual(STATE.get(k), v)


class InnerC(unittest.TestCase):
    layer = LayerC

    def test(self):
        expect = {
            "base": "setup",
            "layerB": "setup",
            "layerC": "setup",
            "layerA": "setup",
            "layerA.test": "setup",
        }
        for k, v in expect.items():
            self.assertEqual(STATE.get(k), v)

    def test2(self):
        expect = {
            "base": "setup",
            "layerB": "setup",
            "layerC": "setup",
            "layerA": "setup",
            "layerA.test": "setup",
        }
        for k, v in expect.items():
            self.assertEqual(STATE.get(k), v)


class InnerD(unittest.TestCase):
    layer = LayerD

    def test(self):
        """test with docstring"""
        self.assertEqual({"base": "setup", "layerD": "setup"}, STATE)


class NoLayer(unittest.TestCase):
    def test(self):
        self.assertEqual(STATE, {})



import unittest

from nose2 import session
from nose2.plugins import dundertest
from nose2.tests._common import TestCase


class TestDunderTestPlugin(TestCase):
    tags = ["unit"]

    def setUp(self):
        class DummyCase(TestCase):
            def test_a(self):
                pass

        self.suite = unittest.TestSuite()
        self.caseClass = DummyCase
        self.session = session.Session()
        self.plugin = dundertest.DunderTestFilter(session=self.session)
        self.plugin.register()

    def test_undefined_dunder_test_attribute_keeps_test(self):
        self.suite.addTest(self.caseClass("test_a"))
        self.plugin.removeNonTests(self.suite)
        self.assertEqual(len(list(self.suite)), 1)

    def test_false_dunder_test_attribute_removes_test(self):
        dummyTest = self.caseClass("test_a")
        dummyTest.__test__ = False
        self.suite.addTest(dummyTest)
        self.plugin.removeNonTests(self.suite)
        self.assertEqual(len(list(self.suite)), 0)



"""
Profile test execution using cProfile.

This plugin implements :func:`startTestRun` and replaces
``event.executeTests`` with :meth:`cProfile.Profile.runcall`. It
implements :func:`beforeSummaryReport` to output profiling information
before the final test summary time. Config file options ``filename``,
``sort`` and ``restrict`` can be used to change where profiling
information is saved and how it is presented.

Load this plugin by running nose2 with the
`--plugin=nose2.plugins.prof` option and activate it
with the `--profile` option,or put the corresponding
entries (`plugin` and `always_on`) in the respective
sections of the configuration file.

"""
import cProfile
import logging
import pstats

from nose2 import events, util

log = logging.getLogger(__name__)
__unittest = True


class Profiler(events.Plugin):

    """Profile the test run"""

    configSection = "profiler"
    commandLineSwitch = ("P", "profile", "Run tests under profiler")

    def __init__(self):
        self.pfile = self.config.as_str("filename", "")
        self.sort = self.config.as_str("sort", "cumulative")
        self.restrict = self.config.as_list("restrict", [])
        self.fileno = None

    def startTestRun(self, event):
        """Set up the profiler"""
        self.prof = cProfile.Profile()
        event.executeTests = self.prof.runcall

    def beforeSummaryReport(self, event):
        """Output profiling results"""

        # write prof output to stream
        class Stream:
            def write(self, *msg):
                for m in msg:
                    event.stream.write(m)
                    event.stream.write(" ")
                event.stream.flush()

        stream = Stream()
        prof_stats = pstats.Stats(self.prof, stream=stream)
        prof_stats.sort_stats(self.sort)
        event.stream.writeln(util.ln("Profiling results"))
        if self.restrict:
            prof_stats.print_stats(*self.restrict)
        else:
            prof_stats.print_stats()

        if self.pfile:
            prof_stats.dump_stats(self.pfile)

        self.prof.disable()
        event.stream.writeln("")



import unittest

from nose2 import config


class TestConfig(unittest.TestCase):
    def setUp(self):
        self.conf = config.Config(
            [("a", " 1 "), ("b", "  x\n  y "), ("c", "0"), ("d", "123")]
        )

    def test_as_int(self):
        self.assertEqual(self.conf.as_int("a"), 1)

    def test_as_str(self):
        self.assertEqual(self.conf.as_str("a"), "1")
        self.assertEqual(self.conf.as_str("b"), "x\n  y")
        self.assertEqual(self.conf.as_str("missing", "default"), "default")

    def test_as_bool(self):
        self.assertEqual(self.conf.as_bool("a"), True)
        self.assertEqual(self.conf.as_bool("c"), False)

    def test_as_float(self):
        self.assertAlmostEqual(self.conf.as_float("a"), 1.0)

    def test_as_list(self):
        self.assertEqual(self.conf.as_list("b"), ["x", "y"])
        self.assertEqual(self.conf.as_list("a"), ["1"])
        self.assertEqual(self.conf.as_list("d"), ["123"])



from nose2 import session
from nose2.events import StartTestRunEvent
from nose2.plugins import prof
from nose2.tests._common import Stub, TestCase


class TestProfPlugin(TestCase):
    tags = ["unit"]

    def setUp(self):
        self.plugin = prof.Profiler(session=session.Session())
        # stub out and save the cProfile and pstats modules
        self.cProfile = prof.cProfile
        self.pstats = prof.pstats
        prof.cProfile = Stub()
        prof.pstats = Stub()

    def tearDown(self):
        prof.cProfile = self.cProfile
        prof.pstats = self.pstats

    def test_startTestRun_sets_executeTests(self):
        _prof = Stub()
        _prof.runcall = object()
        prof.cProfile.Profile = lambda: _prof
        event = StartTestRunEvent(
            runner=None, suite=None, result=None, startTime=None, executeTests=None
        )
        self.plugin.startTestRun(event)
        assert event.executeTests is _prof.runcall, "executeTests was not replaced"



import unittest


class Layer:
    @classmethod
    def setUp(cls):
        pass

    @classmethod
    def testTearDown(cls):
        raise RuntimeError("Bad Error in Layer testTearDown!")


class Test(unittest.TestCase):
    layer = Layer

    def testPass(self):
        pass



"""
Stop the test run after the first error or failure.

This plugin implements :func:`testOutcome` and sets
``event.result.shouldStop`` if it sees an outcome with exc_info that
is not expected.

"""

from nose2 import events

__unittest = True


class FailFast(events.Plugin):

    """Stop the test run after error or failure"""

    commandLineSwitch = (
        "F",
        "fail-fast",
        "Stop the test run after the first error or failure",
    )

    def resultCreated(self, event):
        """Mark new result"""
        if hasattr(event.result, "failfast"):
            event.result.failfast = True

    def testOutcome(self, event):
        """Stop on unexpected error or failure"""
        if event.exc_info and not event.expected:
            event.result.shouldStop = True



import unittest

from lib20171102.mod1 import foo, func


class TestCase(unittest.TestCase):
    def test1(self):
        self.assertEqual(foo, 1)

    def test2(self):
        self.assertEqual(func(), 2)



import sys

from nose2 import session
from nose2.tests._common import FunctionalTestCase, support_file


class SessionFunctionalTests(FunctionalTestCase):
    def setUp(self):
        self.s = session.Session()
        self.s.loadConfigFiles(
            support_file("cfg", "a.cfg"), support_file("cfg", "b.cfg")
        )
        sys.path.insert(0, support_file("lib"))

    def test_session_can_load_config_files(self):
        assert self.s.config.has_section("a")
        assert self.s.config.has_section("b")

    def test_session_holds_plugin_config(self):
        plug_config = self.s.get("a")
        assert plug_config

    def test_session_can_load_plugins_from_modules(self):
        self.s.loadPlugins()
        assert self.s.plugins
        plug = self.s.plugins[0]
        self.assertEqual(plug.a, 1)

    def test_session_config_cacheing(self):
        """Test caching of config sections works"""

        # Create new session (generic one likely already cached
        # depending on test order)
        cache_sess = session.Session()
        cache_sess.loadConfigFiles(support_file("cfg", "a.cfg"))

        # First access to given section, should read from config file
        firstaccess = cache_sess.get("a")
        assert firstaccess.as_int("a") == 1

        # Hack cached Config object internals to make the stored value
        # something different
        cache_sess.configCache["a"]._mvd["a"] = "0"
        newitems = []
        for item in cache_sess.configCache["a"]._items:
            if item != ("a", "1"):
                newitems.append(item)
            else:
                newitems.append(("a", "0"))
        cache_sess.configCache["a"]._items = newitems

        # Second access to given section, confirm returns cached value
        # rather than parsing config file again
        secondaccess = cache_sess.get("a")
        assert secondaccess.as_int("a") == 0



import types

from docutils import nodes
from docutils.parsers.rst import Directive, directives
from docutils.statemachine import ViewList

from nose2 import events, plugins, session, util

AD = "<autodoc>"
__unittest = True


class AutoPlugin(Directive):
    required_arguments = 1
    optional_arguments = 1
    final_argument_whitespace = False
    has_content = False
    option_spec = {"module": directives.unchanged}

    def run(self):
        plugin_name = self.arguments[0]
        parent, plugin = util.object_from_name(plugin_name)
        if isinstance(plugin, types.ModuleType):
            # document all plugins in module
            module = plugin
            mod_name = module.__name__
            plugins = self.plugins(module)

        else:
            if "module" in self.options:
                mod_name = self.options["module"]
            else:
                mod_name = plugin_name[0 : plugin_name.index(plugin.__name__) - 1]
            plugins = [plugin]

        rst = ViewList()
        if mod_name:
            rst.append(".. automodule :: %s\n" % mod_name, AD)
            rst.append("", AD)

        for plug in plugins:
            self.document(rst, plug)

        # parse rst and generate new nodelist
        state = self.state
        node = nodes.section()
        node.document = state.document
        surrounding_title_styles = state.memo.title_styles
        surrounding_section_level = state.memo.section_level
        state.memo.title_styles = []
        state.memo.section_level = 0
        state.nested_parse(rst, 0, node, match_titles=1)
        state.memo.title_styles = surrounding_title_styles
        state.memo.section_level = surrounding_section_level

        return node.children

    def document(self, rst, plugin):
        ssn = session.Session()
        ssn.configClass = ssn.config = config = ConfigBucket()
        ssn.pluginargs = opts = OptBucket()
        plugin_name = plugin.__name__
        config = ssn.config
        obj = plugin(session=ssn)
        try:
            obj.pluginsLoaded(events.PluginsLoadedEvent([obj]))
        except AttributeError:
            pass

        # config options
        if config.vars:
            self.add_config(rst, config, plugin)

        # command-line options
        if opts.opts:
            self.headline(rst, "Command-line options")
            for opt in opts:
                for line in opt.options():
                    rst.append(line, AD)
                rst.append("", AD)

        # class __doc__
        self.headline(rst, "Plugin class reference: %s" % plugin_name)
        rst.append(".. autoclass :: %s" % plugin_name, AD)
        rst.append("   :members:", AD)
        rst.append("", AD)

    def add_config(self, rst, config, plugin):
        # add docs on enabling non-default plugins
        plugin_modname = plugin.__module__
        if plugin_modname not in plugins.DEFAULT_PLUGINS:
            self.headline(rst, "Enable this Plugin")
            rst.append("This plugin is built-in, but not loaded by default.", AD)
            rst.append("", AD)
            rst.append(
                "Even if you specify ``always-on = True`` in the "
                "configuration, it will not run unless you also enable it. "
                "You can do so by putting the following in a "
                ":file:`unittest.cfg` or :file:`nose2.cfg` file",
                AD,
            )
            rst.append("", AD)
            rst.append(".. code-block:: ini", AD)
            rst.append("", AD)
            rst.append("  [unittest]", AD)
            rst.append("  plugins = %s" % plugin_modname, AD)
            rst.append("", AD)
            rst.append(
                "The ``plugins`` parameter may contain a list of plugin "
                "names, including ``%s``" % plugin_modname,
                AD,
            )
            rst.append("", AD)

        headline = "Configuration [%s]" % config.section
        self.headline(rst, headline)

        for var in sorted(config.vars.keys()):
            info = config.vars[var]
            rst.append(".. rst:configvar :: %s" % var, AD)
            rst.append("  ", AD)
            rst.append("  :Default: {default}".format(**info), AD)
            rst.append("  :Type: {type}".format(**info), AD)
            rst.append("", AD)

        self.headline(rst, "Sample configuration", "-")
        rst.append(
            "The default configuration is equivalent to including "
            "the following in a :file:`unittest.cfg` file.",
            AD,
        )
        rst.append("", AD)
        rst.append(".. code-block:: ini", AD)
        rst.append("  ", AD)
        rst.append("  [%s]" % config.section, AD)
        for var in sorted(config.vars.keys()):
            info = config.vars[var]
            entry = "  %s = " % (var)
            if info["type"] == "list":
                if info["default"]:
                    pad = " " * len(entry)
                    entry = "{}{}".format(entry, info["default"][0])
                    rst.append(entry, AD)
                    for val in info["default"][1:]:
                        rst.append(f"{pad}{val}", AD)
                else:
                    rst.append(entry, AD)
            elif info["default"] is not None:
                entry = "{}{}".format(entry, info["default"])
                rst.append(entry, AD)
        rst.append("", AD)

    def headline(self, rst, headline, level="="):
        rst.append(headline, AD)
        rst.append(level * len(headline), AD)
        rst.append("", AD)

    def plugins(self, module):
        for _, item in util.iter_attrs(module):
            if not isinstance(item, type):
                continue
            if issubclass(item, events.Plugin):
                yield item


def setup(app):
    app.add_directive("autoplugin", AutoPlugin)
    app.add_object_type("configvar", "config", "pair: %s; configvar")


DEFAULT = object()


class ConfigBucket:
    def __init__(self):
        self.section = None
        self.vars = {}

    def __call__(self, items):
        self.vars = dict(items)
        return self

    def has_section(self, section):
        self.section = section
        return False

    def items(self):
        return self.vars.items()

    def as_bool(self, item, default=DEFAULT):
        self.vars[item] = {"type": "boolean", "default": default}
        return default

    as_tri = as_bool

    def as_int(self, item, default=DEFAULT):
        self.vars[item] = {"type": "integer", "default": default}
        return default

    def as_float(self, item, default=DEFAULT):
        self.vars[item] = {"type": "float", "default": default}
        return default

    def as_str(self, item, default=DEFAULT):
        self.vars[item] = {"type": "str", "default": default}
        return default

    def as_list(self, item, default=DEFAULT):
        self.vars[item] = {"type": "list", "default": default}
        return default

    def __getitem__(self, item):
        self.vars[item] = {"type": None, "default": DEFAULT}

    def get(self, item, default=DEFAULT):
        self.vars[item] = {"type": None, "default": default}
        return default


class OptBucket:
    def __init__(self, doc=None, prog="nosetests"):
        self.seen = set()
        self.opts = []
        self.doc = doc
        self.prog = prog

    def __iter__(self):
        return iter(self.opts)

    def format_help(self):
        return self.doc.replace("%prog", self.prog).replace(":\n", "::\n")

    def add_argument(self, *arg, **kw):
        if arg not in self.seen:
            self.opts.append(Opt(*arg, **kw))
            self.seen.add(arg)

    def __call__(self, callback, opt=None, longOpt=None, help=None):
        opts = []
        if opt is not None:
            opts.append("-" + opt)
        if longOpt is not None:
            opts.append("--" + longOpt)
        self.add_option(*opts, help=help)


class Opt:
    def __init__(self, *arg, **kw):
        self.opts = arg
        self.action = kw.pop("action", None)
        self.default = kw.pop("default", None)
        self.metavar = kw.pop("metavar", None)
        self.help = kw.pop("help", None)

    def options(self):
        buf = []
        for optstring in self.opts:
            desc = optstring
            if self.action not in ("store_true", "store_false", None):
                desc += " %s" % self.meta(optstring)
            buf.append(desc)
            res = [".. cmdoption :: " + ", ".join(buf)]
            if self.help:
                res.append("")
                res.append("   %s" % self.help)
            res.append("")
        return res

    def meta(self, optstring):
        # FIXME optparser default metavar?
        return self.metavar or "DEFAULT"



# Adapted from unittest2/loader.py from the unittest2 plugins branch.
# This module contains some code copied from unittest2/loader.py and other
# code developed in reference to that module and others within unittest2.
# unittest2 is Copyright (c) 2001-2010 Python Software Foundation; All
# Rights Reserved. See: http://docs.python.org/license.html

import logging
import traceback
import unittest

from nose2 import events

log = logging.getLogger(__name__)
__unittest = True


class PluggableTestLoader:

    """Test loader that defers all loading to plugins

    :param session: Test run session.

    .. attribute :: suiteClass

       Suite class to use. Default: :class:`unittest.TestSuite`.

    """

    suiteClass = unittest.TestSuite

    def __init__(self, session):
        self.session = session

    def loadTestsFromModule(self, module):
        """Load tests from module.

        Fires :func:`loadTestsFromModule` hook.

        """
        evt = events.LoadFromModuleEvent(self, module)
        result = self.session.hooks.loadTestsFromModule(evt)
        if evt.handled:
            suite = result or self.suiteClass()
        else:
            suite = self.suiteClass(evt.extraTests)
        filterevt = events.ModuleSuiteEvent(self, module, suite)
        result = self.session.hooks.moduleLoadedSuite(filterevt)
        if result:
            return result or self.suiteClass()
        return filterevt.suite

    def loadTestsFromNames(self, testNames, module=None):
        """Load tests from test names.

        Fires :func:`loadTestsFromNames` hook.

        """
        event = events.LoadFromNamesEvent(self, testNames, module)
        result = self.session.hooks.loadTestsFromNames(event)
        log.debug("loadTestsFromNames event %s result %s", event, result)
        if event.handled:
            suites = result or []
        else:
            if event.names:
                suites = [self.loadTestsFromName(name, module) for name in event.names]
            elif module:
                suites = self.loadTestsFromModule(module)
        if event.extraTests:
            suites.extend(event.extraTests)
        return self.suiteClass(suites)

    def loadTestsFromName(self, name, module=None):
        """Load tests from test name.

        Fires :func:`loadTestsFromName` hook.

        """
        log.debug("loadTestsFromName %s/%s", name, module)
        event = events.LoadFromNameEvent(self, name, module)
        result = self.session.hooks.loadTestsFromName(event)
        if event.handled:
            suite = result or self.suiteClass()
            return suite
        return self.suiteClass(event.extraTests)

    def failedImport(self, name):
        """Make test case representing a failed import."""
        message = f"Failed to import test module: {name}\n{traceback.format_exc()}"
        return self._makeFailedTest("ModuleImportFailure", name, ImportError(message))

    def failedLoadTests(self, name, exception):
        """Make test case representing a failed test load."""
        return self._makeFailedTest("LoadTestsFailure", name, exception)

    def sortTestMethodsUsing(self, name):
        """Sort key for test case test methods."""
        return name.lower()

    def discover(self, start_dir=None, pattern=None):
        """Compatibility shim for ``load_tests`` protocol."""
        try:
            oldsd = self.session.startDir
            self.session.startDir = start_dir
            return self.loadTestsFromNames([])
        finally:
            self.session.startDir = oldsd

    def _makeFailedTest(self, classname, methodname, exception):
        if isinstance(exception, Exception):

            def testFailure(self):
                raise exception

        elif isinstance(exception, tuple) and len(exception) == 3:
            _, value, tb = exception

            def testFailure(self):
                raise value.with_traceback(tb)

        else:  # unreachable
            raise NotImplementedError

        attrs = {methodname: testFailure}
        TestClass = type(classname, (unittest.TestCase,), attrs)
        return self.suiteClass((TestClass(methodname),))

    def __repr__(self):
        return "<%s>" % self.__class__.__name__



"""
Load tests from :class:`unittest.TestCase` subclasses.

This plugin implements :func:`loadTestsFromName` and
:func:`loadTestsFromModule` to load tests from
:class:`unittest.TestCase` subclasses found in modules or named on the
command line.


"""
# Adapted from unittest2/loader.py from the unittest2 plugins branch.
# This module contains some code copied from unittest2/loader.py and other
# code developed in reference to that module and others within unittest2.
# unittest2 is Copyright (c) 2001-2010 Python Software Foundation; All
# Rights Reserved. See: http://docs.python.org/license.html

import inspect
import logging
import sys
import unittest

from nose2 import events, util

__unittest = True


log = logging.getLogger(__name__)


class TestCaseLoader(events.Plugin):

    """Loader plugin that loads from test cases"""

    alwaysOn = True
    configSection = "testcases"

    def registerInSubprocess(self, event):
        event.pluginClasses.append(self.__class__)

    def loadTestsFromModule(self, event):
        """Load tests in :class:`unittest.TestCase` subclasses"""
        seen = set()
        module = event.module
        for _, obj in util.iter_attrs(module):
            if id(obj) in seen:
                continue
            seen.add(id(obj))
            if isinstance(obj, type) and issubclass(obj, unittest.TestCase):
                event.extraTests.append(self._loadTestsFromTestCase(event, obj))

    def loadTestsFromName(self, event):
        """Load tests from event.name if it names a test case/method"""
        name = event.name
        module = event.module
        log.debug("load %s from %s", name, module)
        try:
            result = util.test_from_name(name, module)
        except (AttributeError, ImportError):
            event.handled = True
            return event.loader.failedLoadTests(name, sys.exc_info())
        if result is None:
            return
        parent, obj, name, index = result
        if isinstance(obj, type) and issubclass(obj, unittest.TestCase):
            # name is a test case class
            event.extraTests.append(self._loadTestsFromTestCase(event, obj))
        elif (
            isinstance(parent, type)
            and issubclass(parent, unittest.TestCase)
            and not inspect.isgeneratorfunction(obj)
            and not hasattr(obj, "paramList")
        ):
            # name is a single test method
            event.extraTests.append(parent(obj.__name__))

    def _loadTestsFromTestCase(self, event, testCaseClass):
        evt = events.LoadFromTestCaseEvent(event.loader, testCaseClass)
        result = self.session.hooks.loadTestsFromTestCase(evt)
        if evt.handled:
            loaded_suite = result or event.loader.suiteClass()
        else:
            names = self._getTestCaseNames(event, testCaseClass)
            if not names and hasattr(testCaseClass, "runTest"):
                names = ["runTest"]
            # FIXME return failure test case if name not in testcase class
            loaded_suite = event.loader.suiteClass(map(testCaseClass, names))
        if evt.extraTests:
            loaded_suite.addTests(evt.extraTests)
        return loaded_suite

    def _getTestCaseNames(self, event, testCaseClass):
        excluded = set()

        def isTestMethod(attrname, testCaseClass=testCaseClass, excluded=excluded):
            prefix = evt.testMethodPrefix or self.session.testMethodPrefix
            return (
                attrname.startswith(prefix)
                and hasattr(getattr(testCaseClass, attrname), "__call__")
                and attrname not in excluded
            )

        evt = events.GetTestCaseNamesEvent(event.loader, testCaseClass, isTestMethod)
        result = self.session.hooks.getTestCaseNames(evt)
        if evt.handled:
            test_names = result or []
        else:
            excluded.update(evt.excludedNames)
            test_names = [entry for entry in dir(testCaseClass) if isTestMethod(entry)]
        if evt.extraNames:
            test_names.extend(evt.extraNames)
        sortkey = getattr(
            testCaseClass, "sortTestMethodsUsing", event.loader.sortTestMethodsUsing
        )
        if sortkey:
            test_names.sort(key=sortkey)
        return test_names



from nose2.tools.params import params

# fmt: off


# multiline function definition
def test_demo(
):
    x = 1
    y = 2
    assert x > y, "oh noez, x <= y"


@params(('foo',),
        ('bar',))
def test_multiline_deco(value):
    assert not value
# fmt: on



# This module contains some code copied from unittest2/ and other code
# developed in reference to unittest2.
# unittest2 is Copyright (c) 2001-2010 Python Software Foundation; All
# Rights Reserved. See: http://docs.python.org/license.html
__unittest = True


class TestNotFoundError(Exception):

    """Raised when a named test cannot be found"""


class LoadTestsFailure(Exception):

    """Raised when a test cannot be loaded"""



import unittest

from nose2 import events, session


class SessionUnitTests(unittest.TestCase):
    def test_can_create_session(self):
        session.Session()

    def test_load_plugins_from_module_can_load_plugins(self):
        class fakemod:
            pass

        f = fakemod()

        class A(events.Plugin):
            pass

        f.A = A
        s = session.Session()
        s.loadPluginsFromModule(f)
        assert s.plugins
        a = s.plugins[0]
        self.assertEqual(a.session, s)

    def test_load_plugins_from_module_does_not_load_plain_Plugins(self):
        class fakemod:
            pass

        f = fakemod()

        f.A = events.Plugin
        s = session.Session()
        s.loadPluginsFromModule(f)
        self.assertEqual(len(s.plugins), 0)

    def test_load_plugins_from_module_does_not_duplicate_always_on_plugins(self):
        class fakemod:
            pass

        f = fakemod()

        class A(events.Plugin):
            alwaysOn = True

        f.A = A
        s = session.Session()
        s.loadPluginsFromModule(f)
        self.assertEqual(len(s.plugins), 1)



import logging
import unittest

import nose2

log = logging.getLogger(__name__)


class Test(unittest.TestCase):
    def test_logging_config(self):
        log.debug("foo")
        log.info("bar")
        assert False


if __name__ == "__main__":
    nose2.main()



import unittest

THINGS = []


def setUpModule():
    THINGS.append(1)


def tearDownModule():
    while THINGS:
        THINGS.pop()


class Test(unittest.TestCase):
    def test_1(self):
        assert THINGS, "setup didn't run"

    def test_2(self):
        assert THINGS, "setup didn't run"



"""
Buffer stdout and/or stderr during test execution, appending any
output to the error reports of failed tests.

This allows you to use print for debugging in tests without making
your test runs noisy.

This plugin implements :func:`startTest`, :func:`stopTest`,
:func:`setTestOutcome`, :func:`outcomeDetail`, :func:`beforeInteraction`
and :func:`afterInteraction` to manage capturing sys.stdout and/or
sys.stderr into buffers, attaching the buffered output to test error
report detail, and getting out of the way when other plugins want to
talk to the user.

"""

import io
import sys

from nose2 import events
from nose2.util import ln

__unittest = True


class _Buffer:
    def __init__(self, stream):
        self._stream = stream
        self._buffer = io.StringIO()

    def fileno(self):
        return self._stream.fileno()

    def __getattr__(self, attr):
        # this happens on unpickling
        if attr == "_buffer":
            raise AttributeError("No _buffer yet")
        return getattr(self._buffer, attr)

    def __le__(self, obj):
        return self._buffer.getvalue() == obj

    def __eq__(self, obj):
        return self._buffer.getvalue() == obj

    def __str__(self):
        return self._buffer.getvalue()

    def __repr__(self):
        return repr(self._buffer.getvalue())


class OutputBufferPlugin(events.Plugin):

    """Buffer output during test execution"""

    commandLineSwitch = ("B", "output-buffer", "Enable output buffer")
    configSection = "output-buffer"

    def __init__(self):
        self.captureStdout = self.config.as_bool("stdout", default=True)
        self.captureStderr = self.config.as_bool("stderr", default=False)
        self.bufStdout = self.bufStderr = None
        self.realStdout = sys.__stdout__
        self.realStderr = sys.__stderr__
        self._disable = False

    def registerInSubprocess(self, event):
        event.pluginClasses.append(self.__class__)
        # turn off in this process: the subproc will run the tests
        self._disable = True

    def startSubprocess(self, event):
        self.realStdout = sys.__stdout__
        self.realStderr = sys.__stderr__

    def startTest(self, event):
        """Start buffering selected stream(s)"""
        self._buffer()

    def stopTest(self, event):
        """Stop buffering"""
        self._restore()

    def _get_stream_unicode_save(self, stream, buffer):
        buf = buffer.getvalue()
        return "\n".join(
            [
                ln(f">> begin captured {stream} <<"),
                buf,
                ln(f">> end captured {stream} <<"),
            ]
        )

    def setTestOutcome(self, event):
        """Attach buffer(s) to event.metadata"""
        if self._disable:
            return
        if self.captureStdout and "stdout" not in event.metadata:
            event.metadata["stdout"] = self._get_stream_unicode_save(
                "stdout", self.bufStdout
            )
        if self.captureStderr and "stderr" not in event.metadata:
            event.metadata["stderr"] = self._get_stream_unicode_save(
                "stderr", self.bufStderr
            )

    def outcomeDetail(self, event):
        """Add buffered output to event.extraDetail"""
        for stream in ("stdout", "stderr"):
            if stream in event.outcomeEvent.metadata:
                b = event.outcomeEvent.metadata[stream]
                if b:
                    event.extraDetail.append(b)

    def beforeInteraction(self, event):
        """Stop buffering so users can see stdout"""
        self._restore()

    def afterInteraction(self, event):
        """Start buffering again (does not clear buffers)"""
        self._buffer(fresh=False)

    def stopSubprocess(self, event):
        self._restore()

    def _restore(self):
        if self._disable:
            return
        if self.captureStdout:
            sys.stdout = self.realStdout
        if self.captureStderr:
            sys.stderr = self.realStderr

    def _buffer(self, fresh=True):
        if self._disable:
            return
        if self.captureStdout:
            if fresh or self.bufStdout is None:
                self.bufStdout = _Buffer(sys.stdout)
            sys.stdout = self.bufStdout
        if self.captureStderr:
            if fresh or self.bufStderr is None:
                self.bufStderr = _Buffer(sys.stderr)
            sys.stderr = self.bufStderr



# This module contains some code copied from unittest2/runner.py and other
# code developed in reference to that module and others within unittest2.
# unittest2 is Copyright (c) 2001-2010 Python Software Foundation; All
# Rights Reserved. See: http://docs.python.org/license.html

import time

from nose2 import events, result

__unittest = True


class PluggableTestRunner:

    """Test runner that defers most work to plugins.

    :param session: Test run session

    .. attribute :: resultClass

       Class to instantiate to create test result. Default:
       :class:`nose2.result.PluggableTestResult`.

    """

    resultClass = result.PluggableTestResult

    def __init__(self, session):
        self.session = session

    def run(self, test):
        """Run tests.

        :param test: A unittest :class:`TestSuite`` or :class:`TestClass`.
        :returns: Test result

        Fires :func:`startTestRun` and :func:`stopTestRun` hooks.

        """
        result = self._makeResult()

        def executor(suite, result):
            return suite(result)

        startTime = time.time()
        event = events.StartTestRunEvent(self, test, result, startTime, executor)
        self.session.hooks.startTestRun(event)

        # allows startTestRun to modify test suite
        test = event.suite
        # ... and test execution
        executor = event.executeTests
        try:
            if not event.handled:
                executor(test, result)
        finally:
            stopTime = time.time()
            timeTaken = stopTime - startTime
            event = events.StopTestRunEvent(self, result, stopTime, timeTaken)
            self.session.hooks.stopTestRun(event)
            self.session.hooks.afterTestRun(event)
        return result

    def _makeResult(self):
        result = self.resultClass(self.session)
        event = events.ResultCreatedEvent(result)
        self.session.hooks.resultCreated(event)
        self.session.testResult = event.result
        return event.result

    def __repr__(self):
        return "<%s>" % self.__class__.__name__



import unittest

from nose2 import events, session
from nose2.plugins import attrib
from nose2.tests._common import TestCase


class TestAttribPlugin(TestCase):
    tags = ["unit"]

    def setUp(self):
        class TC_1(TestCase):
            tags = ["a", "b"]

            def test_a(self):
                pass

            test_a.a = 1
            test_a.c = 0

            def test_b(self):
                pass

            test_b.b = 1

        self.TC_1 = TC_1
        self.session = session.Session()
        self.plugin = attrib.AttributeSelector(session=self.session)
        self.plugin.register()

    def test_validate_attribs_with_simple_values(self):
        assert self.plugin.validateAttrib(self.TC_1("test_a"), [[("a", "1")]])
        assert self.plugin.validateAttrib(self.TC_1("test_a"), [[("a", True)]])
        assert self.plugin.validateAttrib(self.TC_1("test_a"), [[("c", False)]])
        assert self.plugin.validateAttrib(self.TC_1("test_b"), [[("b", "1")]])
        assert not self.plugin.validateAttrib(self.TC_1("test_a"), [[("a", False)]])
        assert not self.plugin.validateAttrib(self.TC_1("test_a"), [[("c", True)]])
        assert not self.plugin.validateAttrib(self.TC_1("test_a"), [[("a", "2")]])
        assert not self.plugin.validateAttrib(self.TC_1("test_a"), [[("b", "1")]])

    def test_validate_attribs_with_callable(self):
        assert self.plugin.validateAttrib(
            self.TC_1("test_a"), [[("a", lambda key, test: True)]]
        )
        assert not self.plugin.validateAttrib(
            self.TC_1("test_a"), [[("a", lambda key, test: False)]]
        )

    def test_validate_attribs_against_list(self):
        assert self.plugin.validateAttrib(self.TC_1("test_a"), [[("tags", "a")]])
        assert self.plugin.validateAttrib(self.TC_1("test_a"), [[("tags", "b")]])
        assert not self.plugin.validateAttrib(self.TC_1("test_a"), [[("tags", "c")]])

    def test_module_loaded_suite_filters_suite(self):
        self.plugin.attribs = ["a"]
        suite = unittest.TestSuite()
        suite.addTest(self.TC_1("test_a"))
        suite.addTest(self.TC_1("test_b"))
        event = events.ModuleSuiteEvent(None, None, suite)
        self.session.hooks.moduleLoadedSuite(event)
        self.assertEqual(len(event.suite._tests), 1)
        self.assertEqual(event.suite._tests[0]._testMethodName, "test_a")



from nose2.tests._common import FunctionalTestCase, windows_ci_skip


class TestPrettyAsserts(FunctionalTestCase):
    def assertProcOutputPattern(self, proc, expected, assert_exit_status=1):
        """
        - proc: a popen proc to check output on
        - expected: expected output pattern
        """
        stdout, stderr = proc.communicate()

        if assert_exit_status is not None:
            self.assertEqual(assert_exit_status, proc.poll())

        self.assertTestRunOutputMatches(proc, stderr=expected)

        return stderr

    def test_simple_global(self):
        proc = self.runIn(
            "scenario/pretty_asserts/simple_global", "-v", "--pretty-assert"
        )
        expected = "\n".join(
            [">>> assert myglob == 2", "", "values:", "    myglob = 1", ""]
        )
        self.assertProcOutputPattern(proc, expected)

    def test_conf_on(self):
        proc = self.runIn("scenario/pretty_asserts/conf_on", "-v")
        expected = "\n".join(
            [">>> assert myglob == 2", "", "values:", "    myglob = 1", ""]
        )
        self.assertProcOutputPattern(proc, expected)

    def test_conf_on_suppresses_clihelp(self):
        proc = self.runIn("scenario/pretty_asserts/conf_on", "--help")
        stdout, stderr = proc.communicate()
        exit_status = proc.poll()
        assert "--pretty-assert" not in stdout
        assert "--pretty-assert" not in stderr
        assert exit_status == 0

    def test_conf_on_plus_arg(self):
        """ensures that #432 stays fixed"""
        proc = self.runIn("scenario/pretty_asserts/conf_on", "-v", "--pretty-assert")
        expected = "\n".join(
            [">>> assert myglob == 2", "", "values:", "    myglob = 1", ""]
        )
        self.assertProcOutputPattern(proc, expected)

    def test_assign_after_assert(self):
        proc = self.runIn(
            "scenario/pretty_asserts/assign_after_assert", "-v", "--pretty-assert"
        )
        expected = "\n".join([">>> assert x == 2", "", "values:", "    x = 1", ""])
        self.assertProcOutputPattern(proc, expected)

    def test_multiline_assert_statement(self):
        proc = self.runIn(
            "scenario/pretty_asserts/multiline_statement", "-v", "--pretty-assert"
        )
        expected = "\n".join(
            [
                ">>> assert \\(x",
                ">>>         >",
                '>>>         y\\), \\"oh noez, x <= y\\"',
                "",
                "message:",
                "    oh noez, x <= y",
                "",
                "values:",
                "    x = 1",
                "    y = 2",
                "",
            ]
        )
        self.assertProcOutputPattern(proc, expected)

    def test_multiline_funcdef(self):
        proc = self.runIn(
            "scenario/pretty_asserts/multiline_funcdef", "--pretty-assert"
        )
        expected1 = "\n".join(
            [
                '>>> assert x > y, \\"oh noez, x <= y\\"',
                "",
                "message:",
                "    oh noez, x <= y",
                "",
                "values:",
                "    x = 1",
                "    y = 2",
                "",
            ]
        )
        expected2 = "\n".join(
            [">>> assert not value", "", "values:", "    value = 'foo'", ""]
        )
        expected3 = "\n".join(
            [">>> assert not value", "", "values:", "    value = 'bar'", ""]
        )
        self.assertProcOutputPattern(proc, expected1)
        self.assertProcOutputPattern(proc, expected2)
        self.assertProcOutputPattern(proc, expected3)

    def test_assert_attribute_resolution(self):
        proc = self.runIn(
            "scenario/pretty_asserts/attribute_resolution", "-v", "--pretty-assert"
        )
        expected = "\n".join(
            [
                ">>> assert x.x \\!= \\(self",
                ">>>                \\).x",
                "",
                "values:",
                (
                    "    x = <test_prettyassert_attribute_resolution.TestFoo"
                    " testMethod=test_ohnoez>"
                ),
                "    x.x = 1",
                (
                    "    self = <test_prettyassert_attribute_resolution.TestFoo"
                    " testMethod=test_ohnoez>"
                ),
                "",
            ]
        )
        self.assertProcOutputPattern(proc, expected)

    @windows_ci_skip
    def test_assert_attribute_resolution2(self):
        proc = self.runIn(
            "scenario/pretty_asserts/attribute_resolution2", "-v", "--pretty-assert"
        )
        expected = "\n".join(
            [
                ">>> assert foo\\(\\).x \\!= self.x",
                "",
                "values:",
                (
                    "    foo = <function (TestFoo.test_ohnoez.<locals>.|)foo at "
                    "0x[a-z0-9]+>"
                ),
                (
                    "    self = <test_prettyassert_attribute_resolution2.TestFoo"
                    " testMethod=test_ohnoez>"
                ),
                "    self.x = 1",
                "",
            ]
        )
        self.assertProcOutputPattern(proc, expected)

    def test_assert_ignore_passing(self):
        proc = self.runIn(
            "scenario/pretty_asserts/ignore_passing", "-v", "--pretty-assert"
        )
        expected1 = "\n".join(
            [">>> assert x; assert y", "", "values:", "    y = False", ""]
        )
        expected2 = "\n".join([">>> assert q", "", "values:", "    q = 0", ""])
        self.assertProcOutputPattern(proc, expected1)
        self.assertProcOutputPattern(proc, expected2)

    def test_unittest_assertion(self):
        proc = self.runIn(
            "scenario/pretty_asserts/unittest_assertion", "-v", "--pretty-assert"
        )
        # look for typical unittest output
        expected = "self.assertTrue\\(x\\)\nAssertionError: False is not true"
        stderr = self.assertProcOutputPattern(proc, expected)
        # the assertion line wasn't reprinted by prettyassert
        self.assertNotIn(">>> self.assertTrue", stderr)
        # the assertion values weren't printed by prettyassert
        self.assertNotIn("values:", stderr)
